""" adfs_auth module definition """

#from . import configuration
#from . import cli
#from . import main

VERSION = "0.3.0"
BUILD_TIME = "2018-01-26 01:25:00"

__title__ = "aws_adfs_auth"
__version__ = VERSION
__summary__ = "aws_adfs_auth command line utility"
__license__ = "APACHE2"
__uri__ = "https://github.com/jschwellach/aws-adfs-auth"
__author__ = "Janos Schwellach"
__email__ = "jschwellach@gmail.com"
